MODULE obc_oce
   !!======================================================================
   !!                       ***  MODULE obc_oce   ***
   !! Unstructured Open Boundary Cond. :   define related variables
   !!======================================================================
   !! History :  1.0  !  2001-05  (J. Chanut, A. Sellar)  Original code
   !!            3.0  !  2008-04  (NEMO team)  add in the reference version     
   !!            3.3  !  2010-09  (D. Storkey) add ice boundary conditions
   !!            3.4  !  2011     (D. Storkey, J. Chanut) OBC-BDY merge
   !!----------------------------------------------------------------------
#if defined key_obc 
   !!----------------------------------------------------------------------
   !!   'key_obc'                      Unstructured Open Boundary Condition
   !!----------------------------------------------------------------------
   USE par_oce         ! ocean parameters
   USE obc_par         ! Unstructured boundary parameters
   USE lib_mpp         ! distributed memory computing

   IMPLICIT NONE
   PUBLIC

   TYPE, PUBLIC ::   OBC_INDEX    !: Indices and weights which define the open boundary
      INTEGER,          DIMENSION(jpbgrd) ::  nblen
      INTEGER,          DIMENSION(jpbgrd) ::  nblenrim
      INTEGER, POINTER, DIMENSION(:,:)   ::  nbi
      INTEGER, POINTER, DIMENSION(:,:)   ::  nbj
      INTEGER, POINTER, DIMENSION(:,:)   ::  nbr
      INTEGER, POINTER, DIMENSION(:,:)   ::  nbmap
      REAL   , POINTER, DIMENSION(:,:)   ::  nbw
      REAL   , POINTER, DIMENSION(:)     ::  flagu
      REAL   , POINTER, DIMENSION(:)     ::  flagv
   END TYPE OBC_INDEX

   TYPE, PUBLIC ::   OBC_DATA     !: Storage for external data
      REAL, POINTER, DIMENSION(:)     ::  ssh
      REAL, POINTER, DIMENSION(:)     ::  u2d
      REAL, POINTER, DIMENSION(:)     ::  v2d
      REAL, POINTER, DIMENSION(:,:)   ::  u3d
      REAL, POINTER, DIMENSION(:,:)   ::  v3d
      REAL, POINTER, DIMENSION(:,:)   ::  tem
      REAL, POINTER, DIMENSION(:,:)   ::  sal
#if defined key_lim2
      REAL, POINTER, DIMENSION(:)     ::  frld
      REAL, POINTER, DIMENSION(:)     ::  hicif
      REAL, POINTER, DIMENSION(:)     ::  hsnif
#endif
   END TYPE OBC_DATA

   !!----------------------------------------------------------------------
   !! Namelist variables
   !!----------------------------------------------------------------------
   CHARACTER(len=80), DIMENSION(jp_obc) ::   cn_coords_file !: Name of obc coordinates file
   CHARACTER(len=80)                    ::   cn_mask_file   !: Name of obc mask file
   !
   LOGICAL, DIMENSION(jp_obc) ::   ln_coords_file           !: =T read obc coordinates from file; 
   !                                                        !: =F read obc coordinates from namelist
   LOGICAL                    ::   ln_mask_file             !: =T read obcmask from file
   LOGICAL, DIMENSION(jp_obc) ::   ln_tides                 !: =T apply tidal harmonic forcing along open boundaries
   LOGICAL                    ::   ln_vol                   !: =T volume correction             
   LOGICAL, DIMENSION(jp_obc) ::   ln_clim                  !: =T obc data files contain climatological data (time-cyclic)
   !
   INTEGER                    ::   nb_obc                   !: number of open boundary sets
   INTEGER, DIMENSION(jp_obc) ::   nn_rimwidth              !: boundary rim width
   INTEGER, DIMENSION(jp_obc) ::   nn_dtactl      	    !: = 0 use the initial state as obc dta ; 
                                                            !: = 1 read it in a NetCDF file
   INTEGER                    ::   nn_volctl                !: = 0 the total volume will have the variability of the surface Flux E-P 
   !                                                        !  = 1 the volume will be constant during all the integration.
   INTEGER, DIMENSION(jp_obc) ::   nn_dyn2d                 ! Choice of boundary condition for barotropic variables (U,V,SSH)
   INTEGER, DIMENSION(jp_obc) ::   nn_dyn3d                 ! Choice of boundary condition for baroclinic velocities 
   INTEGER, DIMENSION(jp_obc) ::   nn_tra                   ! Choice of boundary condition for active tracers (T and S)
#if defined key_lim2
   INTEGER, DIMENSION(jp_obc) ::   nn_ice_lim2              ! Choice of boundary condition for sea ice variables 
#endif
   !
   INTEGER, DIMENSION(jp_obc) ::   nn_dmp2d_in              ! Damping timescale (days) for 2D solution for inward radiation or FRS 
   INTEGER, DIMENSION(jp_obc) ::   nn_dmp2d_out             ! Damping timescale (days) for 2D solution for outward radiation 
   INTEGER, DIMENSION(jp_obc) ::   nn_dmp3d_in              ! Damping timescale (days) for 3D solution for inward radiation or FRS 
   INTEGER, DIMENSION(jp_obc) ::   nn_dmp3d_out             ! Damping timescale (days) for 3D solution for outward radiation

   
   !!----------------------------------------------------------------------
   !! Global variables
   !!----------------------------------------------------------------------
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:) ::   obctmask   !: Mask defining computational domain at T-points
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:) ::   obcumask   !: Mask defining computational domain at U-points
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:) ::   obcvmask   !: Mask defining computational domain at V-points

   REAL(wp)                                    ::   obcsurftot !: Lateral surface of unstructured open boundary

   !!----------------------------------------------------------------------
   !! open boundary data variables
   !!----------------------------------------------------------------------

   REAL(wp), ALLOCATABLE, DIMENSION(:,:,:)       ::   dta_global        !: workspace for reading in global data arrays
   TYPE(OBC_INDEX), DIMENSION(jp_obc), TARGET    ::   idx_obc           !: obc indices (local process)
   TYPE(OBC_DATA) , DIMENSION(jp_obc)            ::   dta_obc           !: obc external data (local process)

   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2011)
   !! $Id$ 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   FUNCTION obc_oce_alloc()
      !!----------------------------------------------------------------------
      USE lib_mpp, ONLY: ctl_warn, mpp_sum
      !
      INTEGER :: obc_oce_alloc
      !!----------------------------------------------------------------------
      !
      ALLOCATE( obctmask(jpi,jpj) , obcumask(jpi,jpj), obcvmask(jpi,jpj),                    &  
         &      STAT=obc_oce_alloc )
         !
      IF( lk_mpp             )   CALL mpp_sum ( obc_oce_alloc )
      IF( obc_oce_alloc /= 0 )   CALL ctl_warn('obc_oce_alloc: failed to allocate arrays.')
      !
   END FUNCTION obc_oce_alloc

#else
   !!----------------------------------------------------------------------
   !!   Dummy module                NO Unstructured Open Boundary Condition
   !!----------------------------------------------------------------------
   LOGICAL ::   ln_tides = .false.  !: =T apply tidal harmonic forcing along open boundaries
#endif

   !!======================================================================
END MODULE obc_oce

