MODULE obcice_lim2
   !!======================================================================
   !!                       ***  MODULE  obcice_lim2  ***
   !! Unstructured Open Boundary Cond. :  Flow Relaxation Scheme applied sea-ice (LIM2)
   !!======================================================================
   !!  History :  3.3  !  2010-09 (D. Storkey)  Original code
   !!----------------------------------------------------------------------
#if defined   key_obc   &&   defined key_lim2
   !!----------------------------------------------------------------------
   !!   'key_obc'            and                 Unstructured Open Boundary Conditions
   !!   'key_lim2'                                                 LIM-2 sea ice model
   !!----------------------------------------------------------------------
   !!   obc_ice_lim_2      : Application of open boundaries to ice
   !!   obc_ice_frs        : Application of Flow Relaxation Scheme
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers variables
   USE ice_2           ! LIM_2 ice variables
   USE dom_oce         ! ocean space and time domain variables 
   USE obc_oce         ! ocean open boundary conditions
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE in_out_manager  ! write to numout file
   USE lib_mpp         ! distributed memory computing
   
   IMPLICIT NONE
   PRIVATE

   PUBLIC   obc_ice_lim_2    ! routine called in sbcmod

   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id: obcice.F90 2715 2011-03-30 15:58:35Z rblod $
   !! Software governed by the CeCILL licence (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE obc_ice_lim_2( kt )
      !!----------------------------------------------------------------------
      !!                  ***  SUBROUTINE obc_ice_lim_2  ***
      !!
      !! ** Purpose : - Apply open boundary conditions for ice (LIM2)
      !!
      !!----------------------------------------------------------------------
      INTEGER, INTENT( in ) :: kt     ! Main time step counter
      !!
      INTEGER               :: ib_obc ! Loop index

      DO ib_obc=1, nb_obc

         SELECT CASE( nn_ice_lim2(ib_obc) )
         CASE(jp_none)
            CYCLE
         CASE(jp_frs)
            CALL obc_ice_frs( idx_obc(ib_obc), dta_idx(ib_obc) )
         CASE DEFAULT
            CALL ctl_stop( 'obc_tra : unrecognised option for open boundaries for T an S' )
         END SELECT
      ENDDO

   END SUBROUTINE obc_ice_lim_2

   SUBROUTINE obc_ice_frs( idx, dta )
      !!------------------------------------------------------------------------------
      !!                 ***  SUBROUTINE obc_ice_frs  ***
      !!                    
      !! ** Purpose : Apply the Flow Relaxation Scheme for sea-ice fields in the case 
      !!              of unstructured open boundaries. Currently only tested for LIM2.
      !! 
      !! Reference : Engedahl H., 1995: Use of the flow relaxation scheme in a three-
      !!             dimensional baroclinic ocean model with realistic topography. Tellus, 365-382.
      !!------------------------------------------------------------------------------
      TYPE(OBC_INDEX), INTENT(in) ::   idx  ! OBC indices
      TYPE(OBC_DATA),  INTENT(in) ::   dta  ! OBC external data
      !!
      INTEGER  ::   jb, jk, jgrd   ! dummy loop indices
      INTEGER  ::   ii, ij         ! local scalar
      REAL(wp) ::   zwgt, zwgt1    ! local scalar
      !!------------------------------------------------------------------------------
      !
      jgrd = 1      ! Everything is at T-points here
      !
      DO jb = 1, idx%nblen(jgrd)
         DO jk = 1, jpkm1
            ii    = idx%nbi(jb,jgrd)
            ij    = idx%nbj(jb,jgrd)
            zwgt  = idx%nbw(jb,jgrd)
            zwgt1 = 1.e0 - idx%nbw(jb,jgrd)
            frld (ii,ij) = ( frld (ii,ij) * zwgt1 + dta%frld (jb) * zwgt ) * tmask(ii,ij,1)     ! Leads fraction 
            hicif(ii,ij) = ( hicif(ii,ij) * zwgt1 + dta%hicif(jb) * zwgt ) * tmask(ii,ij,1)     ! Ice depth 
            hsnif(ii,ij) = ( hsnif(ii,ij) * zwgt1 + dta%hsnif(jb) * zwgt ) * tmask(ii,ij,1)     ! Snow depth
         END DO
      END DO 
      CALL lbc_lnk( frld, 'T', 1. )                                         ! lateral boundary conditions
      CALL lbc_lnk( hicif, 'T', 1. )   ;   CALL lbc_lnk( hsnif, 'T', 1. )
      !      
   END SUBROUTINE obc_ice_frs
#else
   !!---------------------------------------------------------------------------------
   !!   Default option                                                    Empty module
   !!---------------------------------------------------------------------------------
CONTAINS
   SUBROUTINE obc_ice_lim_2( kt )      ! Empty routine
      WRITE(*,*) 'obc_ice_frs: You should not have seen this print! error?', kt
   END SUBROUTINE obc_ice_lim_2
#endif

   !!=================================================================================
END MODULE obcice_lim2
